#ifndef __types_h__
#define __types_h__

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <assert.h>

#include "config.h"

#define KB (1024)
#define MB (KB*KB)
#define min(a,b) (((a)<(b)) ? (a) : (b))
#define max(a,b) (((a)>(b)) ? (a) : (b))
#define isNotLater(a, b) (((a).tv_sec < (b).tv_sec) || (((a).tv_sec == (b).tv_sec) && ((a).tv_usec <= (b).tv_usec)))
#define panic(code, a...) do { exitCode = (code); fprintf(stderr, a); fprintf(stderr, "\n"); abort(); } while (0)

#define ERR_PARAM      1
#define ERR_INTERNAL   2
#define ERR_PRIVILEGE  3
#define ERR_NOTFOUND   4
#define ERR_EXTCMD     5
#define ERR_CANTSEND   6
#define ERR_CANTREAD   7
#define ERR_CANTBIND   8
#define ERR_SELECT     9
#define ERR_LIBIPQ    10
#define ERR_TIMEOUT   11
#define ERR_ADDRINUSE 12
#define ERR_FILEIO    13

typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned int uint32;

extern volatile int exitCode;

void setVerbosity(int level);
int getVerbosity();
void verbose(int level, const char *format, ...);
char *renderAddress(struct sockaddr_in *addr, char *buf);
char *renderIP(in_addr_t ip, char *buf);
int parseAddressAndPort(const char *string, struct sockaddr_in *returnValue);
in_addr_t getIP(const char *ifName);
void findUnusedPorts(in_addr_t ip, uint16 *ports, int num);

#endif /* defined(__types_h__) */
