#ifndef __proxies_h__
#define __proxies_h__

#include "probes.h"

class PacketTrace;

class Proxy {
protected:
  ProbePolicy *policy;
  PacketTrace *packetTrace;
  int forcedMTU;
  
public:
  struct addressBlock {
    struct sockaddr_in localAddress;
    struct sockaddr_in remoteAddress;
    struct sockaddr_in senderAddress;
    struct sockaddr_in senderPeerAddress;
    struct sockaddr_in receiverAddress;
    struct sockaddr_in receiverPeerAddress;
  };

  Proxy() { this->policy = NULL; this->packetTrace = NULL; this->forcedMTU = -1; };
  virtual ~Proxy() {};
  virtual void getAddresses(struct sockaddr_in *senderLocal, struct sockaddr_in *senderRemote, struct sockaddr_in *receiverLocal, struct sockaddr_in *receiverRemote) { assert(false); };
  virtual void setPacketTrace(PacketTrace *trace) { this->packetTrace = trace; };
  virtual void setProbePolicy(ProbePolicy *policy) { this->policy = policy; };
  virtual void forwardPacketToReceiver(Packet *packet) { assert(false); };
  virtual void forwardPacketToSender(Packet *packet) { assert(false); };
  virtual void sendPacketToRemote(Packet *packet) { assert(false); };
  virtual void setMTU(int mtu) { this->forcedMTU = mtu; };
  virtual void init(struct addressBlock *addresses, const char *deviceName) { assert(false); };
  virtual void run() { assert(false); };
  virtual const char *getName() { return "unknown"; };
};  

#ifdef HAVE_IPQ
class NetfilterProxy : public Proxy {
protected:
  const static int ipqBufferSizeBytes = 1*MB;
  struct sockaddr_in localAddress;
  struct sockaddr_in remoteAddress;
  struct sockaddr_in senderAddress;
  struct sockaddr_in receiverAddress;
  char networkDeviceName[200];
  void *ipqHandle;
  int captureSocket;
  int sendSocket;
  int senderPort;
  int receiverPort;
  int rulesEstablished;
  int rawSocket;
  bool initialized;
  
  void removeFilterRules();
  void sendRaw(Packet *p);
  
public:
  NetfilterProxy();
  virtual ~NetfilterProxy();
  virtual void getAddresses(struct sockaddr_in *senderLocal, struct sockaddr_in *senderRemote, struct sockaddr_in *receiverLocal, struct sockaddr_in *receiverRemote);
  virtual const char *getName() { return "Netfilter"; };
  virtual void init(struct addressBlock *addresses, const char *deviceName);
  virtual void run();
  virtual void forwardPacketToReceiver(Packet *packet);
  virtual void forwardPacketToSender(Packet *packet);
  virtual void sendPacketToRemote(Packet *packet);
};
#endif /* defined(HAVE_IPQ) */

class TunProxy : public Proxy {
protected:
  char networkDeviceName[200];
  char tunDeviceName[200];
  struct addressBlock addresses;
  bool senderHostRouteActive;
  bool receiverHostRouteActive;
  int captureSocket;
  int sendSocket;
  bool initialized;
  int tunHandle;

  void setupTUN(struct addressBlock *addresses);
  void sendRaw(Packet *p);
  
public:
  TunProxy();
  virtual ~TunProxy();
  virtual void getAddresses(struct sockaddr_in *senderLocal, struct sockaddr_in *senderRemote, struct sockaddr_in *receiverLocal, struct sockaddr_in *receiverRemote);
  virtual void forwardPacketToReceiver(Packet *packet);
  virtual void forwardPacketToSender(Packet *packet);
  virtual void sendPacketToRemote(Packet *packet);
  virtual void init(struct addressBlock *addresses, const char *deviceName);
  virtual void run();
  virtual const char *getName() { return "TUN"; };
};  

class PlanetLabProxy : public Proxy {
protected:
  char networkDeviceName[200];
  char tapDeviceName[200];
  char tapHwAddress[6];
  struct addressBlock addresses;
  int captureSocket;
  int sendSocket;
  bool initialized;
  int tapHandle;
  uint16 lastport;

  void setupTAP(struct addressBlock *addresses);
  void sendRaw(Packet *p);
  void writeToTap(Packet *p);

public:
  PlanetLabProxy();
  virtual ~PlanetLabProxy();
  virtual void getAddresses(struct sockaddr_in *senderLocal, struct sockaddr_in *senderRemote, struct sockaddr_in *receiverLocal, struct sockaddr_in *receiverRemote);
  virtual void forwardPacketToReceiver(Packet *packet);
  virtual void forwardPacketToSender(Packet *packet);
  virtual void sendPacketToRemote(Packet *packet);
  virtual void init(struct addressBlock *addresses, const char *deviceName);
  virtual void run();
  virtual const char *getName() { return "PlanetLab"; };
};  

#endif /* defined(__proxies_h__) */
