#ifndef __traces_h__
#define __traces_h__

class Packet;

class KernelTrace {
protected:
  const static int snapshotsPerBlock = 100;
  struct commentRecord {
    struct commentRecord *next;
    char text[1024];
  };
  struct snapshotRecord {
    struct timeval time;
    struct tcp_info state;
  };
  struct snapshotBlock {
    int snapshotsHere;
    struct snapshotBlock *nextBlock;
    snapshotRecord snapshot[snapshotsPerBlock];
  };
  struct snapshotBlock *firstBlock, *lastBlock;
  struct commentRecord *firstComment, *lastComment;
  char outFileName[200];
    
  void writeSnapshot(struct snapshotRecord *record, FILE *stream);
  
public:
  KernelTrace();
  virtual ~KernelTrace();
  virtual void setFileName(const char *filename) { strncpy(outFileName, filename, sizeof(outFileName)); };
  virtual void addSnapshot(struct timeval *time, struct tcp_info *state);
  virtual void addComment(const char *format, ...);
  virtual void finalize();
};

class PacketTrace {
protected:
  char outFileName[200];
  
public:
  PacketTrace() { strcpy(outFileName, "packettrace.out"); };
  virtual ~PacketTrace() {};
  virtual void setFileName(const char *filename) { strncpy(outFileName, filename, sizeof(outFileName)); };
  virtual void init() {};  
  virtual void addPacket(struct timeval *time, Packet *packet) { assert(false); };
  virtual void addComment(const char *format, ...) { assert(false); };
  virtual void finalize() {};
  virtual const char *getName() { return "unknown"; };
  virtual const char *getDefaultExtension() { return ""; };
};

class MonarchPacketTrace : public PacketTrace {
protected:
  const static int maxHeaderLength = 100;
  const static int packetsPerBlock = 100;
  struct commentRecord {
    struct commentRecord *next;
    char text[1024];
  };
  struct packetRecord {
    struct timeval time;
    uint8 headers[100];
    int headerLen;
  };
  struct packetBlock {
    int packetsHere;
    struct packetBlock *nextBlock;
    packetRecord packet[packetsPerBlock];
  };
  struct packetBlock *firstBlock, *lastBlock;
  struct commentRecord *firstComment, *lastComment;
    
  void writePacket(struct packetRecord *packet, FILE *stream);
  
public:
  MonarchPacketTrace();
  virtual ~MonarchPacketTrace();
  virtual void addPacket(struct timeval *time, Packet *packet);
  virtual void addComment(const char *format, ...);
  virtual void init() {};  
  virtual void finalize();
  virtual const char *getName() { return "mout"; };
  virtual const char *getDefaultExtension() { return ".mout"; };
};

#ifdef HAVE_PCAP
#include <pcap.h>

class PcapPacketTrace : public PacketTrace {
protected:
  pcap_dumper_t *myDumper;
  pcap_t *myPcap;
  
public:
  PcapPacketTrace();
  virtual void addPacket(struct timeval *time, Packet *packet);
  virtual void addComment(const char *format, ...);
  virtual void init();  
  virtual void finalize();
  virtual const char *getName() { return "pcap"; };
  virtual const char *getDefaultExtension() { return ".pcap"; };
};
#endif /* defined(have_pcap) */

#endif /* defined(__traces_h__) */
