#include <stdlib.h>
#include <string.h>

#include "monarch.h"
#include "probes.h"
#include "proxies.h"

#define dprintf(a...) do { } while (0)
//#define dprintf(a...) printf(a)

UdpDataPolicy::UdpDataPolicy() : ProbePolicy()
{
  this->nextSequenceNo = 0;
  this->packetTable = new Hashtable();
}

void UdpDataPolicy::receivePacketFromSender(Packet *packet)
{
  dprintf("Receive packet from sender\n");
  int thisSequenceNo = nextSequenceNo++;
  packetTable->put(thisSequenceNo, new Packet(packet));
  proxy->sendPacketToRemote(makeUdpDataPacket(packet, thisSequenceNo));
  probeTrace->probeSent(thisSequenceNo);
}

void UdpDataPolicy::receivePacketFromRemote(Packet *packet)
{
  dprintf("Receive packet from remote\n");
  struct IpHeader *iph = (struct IpHeader*)packet->getBuffer();
  if (iph->ip_proto != IPPROTO_ICMP)
    return;
    
  uint8 *icmpHeader = &(packet->getBuffer()[packet->getIpHeaderLength()]);
  if ((icmpHeader[0] != 3) || (icmpHeader[1] != 3))
    return;
    
  int thisIndex = *((uint16*)&icmpHeader[12]);
  Packet *savedPacket = (Packet*) packetTable->get(thisIndex);
  
  if (!savedPacket)
    panic(ERR_INTERNAL, "UdpDataPolicy: No saved packet found for incoming IcmpPortUnreachable #%d", thisIndex);
  
  probeTrace->responseReceived(thisIndex, packet->getIpId());
  proxy->forwardPacketToReceiver(savedPacket);
}

void UdpDataPolicy::receivePacketFromReceiver(Packet *packet)
{
  dprintf("Receive packet from receiver\n");
  proxy->forwardPacketToSender(packet);
}

Packet *UdpDataPolicy::makeUdpDataPacket(Packet *packet, int sequenceNo)
{
  Packet *probe = new Packet();
  
  struct UdpHeader *udp = (struct UdpHeader*) &(probe->getBuffer()[20]);
  struct IpHeader *ip = (struct IpHeader*) probe->getBuffer();
  uint16 minlen = sizeof(*ip) + sizeof(*udp) + sizeof(uint16);
  uint16 iplen = packet->getIpLength();
  if (iplen < minlen)
    iplen = minlen;
  probe->setLength(iplen);

  memset(probe->getBuffer(), 0, probe->getBufferLen());
  ip->ip_proto = IPPROTO_UDP; 
  ip->ip_ttl = 60; 
  ip->ip_tos = 0; 
  ip->ip_vhl = 0x45; 
  ip->ip_off = htons(IP_DF); 
  ip->ip_len = htons(iplen);
  ip->ip_id = sequenceNo;

  udp->sourcePort = 4711;
  udp->destPort = 10334;
  udp->length = htons(iplen - sizeof(*ip));
  udp->checksum = 0;
  
  return probe;
}
