#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "monarch.h"
#include "probes.h"

ProbeTrace::ProbeTrace()
{
  this->probesSent = 0;
  this->responsesReceived = 0;
  this->receiveSequence = (int*)malloc(sizeof(int) * maxReceived);
  this->ipidSequence = (int*)malloc(sizeof(int) * maxReceived);
  assert(receiveSequence && ipidSequence);
}

ProbeTrace::ProbeTrace(const char *moutFileName)
{
  FILE *infile = fopen(moutFileName, "r");
  if (!infile)
    panic(ERR_INTERNAL, "ProbeTrace: Cannot read mout trace '%s'", moutFileName);
  
  this->probesSent = 0;
  this->responsesReceived = 0;
  this->receiveSequence = (int*)malloc(sizeof(int) * maxReceived);
  this->ipidSequence = (int*)malloc(sizeof(int) * maxReceived);
  assert(receiveSequence && ipidSequence);
  
  char buf[1000];
  while (fgets(buf, sizeof(buf), infile)) {
    char *arg[21];
    arg[0] = strtok(buf, " \r\n");
    for (int i=1; i<21; i++)
      arg[i] = arg[i-1] ? strtok(NULL, " \r\n") : NULL;
    if (!arg[20])
      continue;
    int seq = atoi(arg[6]) - 7777;
    int ack = atoi(arg[8]) - 7777;
    if ((seq<0) || (seq>500))
      continue;
    if (!strcmp(arg[4], "A") && (seq == ack)) {
      probeSent(seq);
    } else if (strchr(arg[4], 'R')) {
      responseReceived(seq, atoi(arg[10]));
    }
  }
  
  fclose(infile);
}


ProbeTrace::~ProbeTrace()
{
  if (receiveSequence)
    free(receiveSequence);
  if (ipidSequence)
    free(ipidSequence);
}

void ProbeTrace::probeSent(int id)
{
  assert(id == probesSent);
  probesSent ++;
}

void ProbeTrace::responseReceived(int id, int ipid)
{
  assert(responsesReceived < maxReceived);
  receiveSequence[responsesReceived] = id;
  ipidSequence[responsesReceived] = ipid;
  responsesReceived ++;
}

void ProbeTrace::dump(FILE *stream)
{
  fprintf(stream, "%d probes sent, %d responses received\n", probesSent, responsesReceived);
  for (int i=0; i<responsesReceived; i++) 
    fprintf(stream, "#%d: seq %d ipid %d\n", i, receiveSequence[i], ipidSequence[i]);
}

void ProbeTrace::snapshot(int *probesSentArg, int *responsesReceivedArg, int **receiveSequenceArg, int **ipidSequenceArg)
{
  *probesSentArg = probesSent;
  *responsesReceivedArg = responsesReceived;
  *receiveSequenceArg = (int*)malloc(responsesReceived * sizeof(int));
  memcpy(*receiveSequenceArg, receiveSequence, responsesReceived * sizeof(int));
  *ipidSequenceArg = (int*)malloc(responsesReceived * sizeof(int));
  memcpy(*ipidSequenceArg, ipidSequence, responsesReceived * sizeof(int));
}
